jQuery(document).ready(function ($) {
    let file_id = ''

    // Clear file name on reload.
    $('#file-filename').val('');

    $('body').on('click', '.fileuploader-input-button', function () {
        $.ajax({
            type: "post",
            dataType: "json",
            url: frontend_uploader.ajax_url,
            data: {'action': 'frontend_create_post', 'nonce': frontend_uploader.frontend_uploader_nonce},
            success: function (response) {
                $('#file-id').val(response.file_id);
                file_id = response.file_id;
            }
        });
    });


    // enable fileuploader plugin
    $('input[name="file-upload"]').fileuploader({
        changeInput: '<div class="fileuploader-input">' +
            '<div class="fileuploader-input-inner">' +
            '<div class="fileuploader-icon-main"></div>' +
            '<h3 class="fileuploader-input-caption"><span>${captions.feedback}</span></h3>' +
            '<p>${captions.or}</p>' +
            '<button type="button" class="fileuploader-input-button"><span>${captions.button}</span></button>' +
            '</div>' +
            '</div>',
        theme: 'dragdrop',
        upload: {
            url: frontend_uploader.ajax_url,
            data: {
                'action': 'frontend_upload_file',
                'nonce': frontend_uploader.frontend_uploader_nonce,
                'file_id': file_id,
                'library': $('#file-upload').attr('data-library'),
                'status': $('#file-upload').attr('data-status'),
                'viewable-role': $('#viewable-role').val(),
                'viewable-email': $('#viewable-email').val(),
                'filename': '',
                'folder': '',
                'selected_library': ''
            },
            type: 'POST',
            enctype: 'multipart/form-data',
            start: true,
            synchron: true,
            beforeSend: function (item, listEl, parentEl, newInputEl, inputEl) {
                item.upload.data.file_id = file_id;

                if ($('#file-filename').length > 0) {
                    item.upload.data.filename = $('#file-filename').val();
                }

                if ($('#file-folder').length > 0) {
                    item.upload.data.folder = $('#file-folder option:selected').val();
                }

                if ($('#file-library').length > 0) {
                    item.upload.data.selected_library = $('#file-library option:selected').val();
                }

                return true;
            },
            onSuccess: function (result, item) {
                result = JSON.parse(result);

                var data = {};

                // get data
                if (result)
                    data = result;
                else
                    data.hasWarnings = true;

                // if success
                if (data.isSuccess && data.files[0]) {
                    item.name = data.files[0].name;
                    item.html.find('.column-title > div:first-child').text(data.files[0].name).attr('title', data.files[0].name);
                }

                // if warnings
                if (data.hasWarnings) {
                    for (var warning in data.warnings) {
                        alert(data.warnings[warning]);
                    }

                    item.html.removeClass('upload-successful').addClass('upload-failed');
                    // go out from success function by calling onError function
                    // in this case we have a animation there
                    // you can also response in PHP with 404
                    return this.onError ? this.onError(item) : null;
                }

                item.html.find('.fileuploader-action-remove').addClass('fileuploader-action-success');

                setTimeout(function () {
                    item.html.find('.progress-bar2').fadeOut(400);
                }, 400);

            },
            onError: function (item) {
                var progressBar = item.html.find('.progress-bar2');

                if (progressBar.length) {
                    progressBar.find('span').html(0 + "%");
                    progressBar.find('.fileuploader-progressbar .bar').width(0 + "%");
                    item.html.find('.progress-bar2').fadeOut(400);
                }

                item.upload.status != 'cancelled' && item.html.find('.fileuploader-action-retry').length == 0 ? item.html.find('.column-actions').prepend(
                    '<button type="button" class="fileuploader-action fileuploader-action-retry" title="Retry"><i class="fileuploader-icon-retry"></i></button>'
                ) : null;
            },
            onProgress: function (data, item) {
                var progressBar = item.html.find('.progress-bar2');

                if (progressBar.length > 0) {
                    progressBar.show();
                    progressBar.find('span').html(data.percentage + "%");
                    progressBar.find('.fileuploader-progressbar .bar').width(data.percentage + "%");
                }
            },
            onComplete: function () {
                $('.filr-frontend-uploader-container').fadeOut(2000);
                $('.filr-frontend-uploader-notice').fadeIn(2000);

                if ('on' === frontend_uploader.reload) {
                    window.location.reload();
                }
            },
        },
        onRemove: function (item) {
            $.ajax({
                type: "post",
                dataType: "json",
                url: frontend_uploader.ajax_url,
                data: {
                    'action': 'frontend_delete_file',
                    'nonce': frontend_uploader.frontend_uploader_nonce,
                    'filename': item.name
                },
            });

        },
        captions: $.extend(true, {}, $.fn.fileuploader.languages['en'], {
            feedback: frontend_uploader.translations[0],
            feedback2: frontend_uploader.translations[0],
            drop: frontend_uploader.translations[0],
            or: frontend_uploader.translations[1],
            button: frontend_uploader.translations[2],
        }),
    });
});