<?php

namespace filr;

if ( class_exists( 'ZipArchive' ) ) {
	/**
	 * Class which handles ZIP file creation.
	 */
	class Filr_ZIP extends \ZipArchive {

		/**
		 * Add directory to zip file
		 *
		 * @param string $location given path.
		 * @param string $name given filename.
		 *
		 * @return void
		 */
		private function add_dir( $location, $name ) {
			if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
				$this->addEmptyDir( $name );
				$this->add_files( $location, $name );
			}
		}

		/**
		 * Create zip file while add directory.
		 *
		 * @param string $location full path.
		 * @param string $name file name.
		 *
		 * @return void
		 */
		private function add_files( $location, $name ) {
			if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
				$name     .= DIRECTORY_SEPARATOR;
				$location .= DIRECTORY_SEPARATOR;
				$dir      = opendir( $location );

				while ( $file = readdir( $dir ) ) {
					if ( '.' == $file || '..' == $file ) {
						continue;
					}
					if ( filetype( $location . $file ) == 'dir' ) {
						$do = 'add_dir';
					} else {
						$do = 'addFile';
					}
					$this->$do( $location . $file, $name . $file );
				}
			}
		}

		/**
		 * Add password encryption to zip file.
		 *
		 * @param string $file_name current file name.
		 * @param string $password given password.
		 *
		 * @return void
		 */
		private function set_zip_password( $file_name, $password ) {
			parent::setEncryptionName( $file_name, parent::EM_AES_256, $password );
		}

		/**
		 * Creates the zip file from file id.
		 *
		 * @param int $file_id current file id.
		 * @param bool $encryption use encryption or not.
		 *
		 * @return string
		 */
		public static function create_zip_file( $file_id, $encryption ) {
			if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
				if ( true === $encryption ) {
					$name = apply_filters( 'filr_zip_file_name', substr( str_shuffle( '1234567890abcefghijklmnopqrstuvwxyz' ), 0, 20 ), $file_id, $encryption );
				} else {
					$name = apply_filters( 'filr_zip_file_name', $file_id, $file_id, $encryption );
				}

				// get file folder.
				$uploads_directory = wp_upload_dir();
				$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
				$file_folder       = $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id;
				$file_name         = $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR . $name . '.zip';
				$file_url          = $uploads_directory['baseurl'] . '/' . $settings['filr_download_directory'] . '/' . $file_id . '/' . $name . '.zip';

				// Remove old zip if exists.
				if ( file_exists( $file_name ) ) {
					unlink( $file_name );
				}

				// ad zip meta record.
				update_post_meta( $file_id, 'file-zip', $file_name );

				// get password for zip file.
				$zip_password = get_post_meta( $file_id, 'zip-password', true );

				// adding it to a zip file.
				$archive = new self();
				$result  = $archive->open( $file_name, \ZipArchive::CREATE );

				if ( true === $result ) {
					$archive->add_dir( $file_folder, $name );

					if ( true === extension_loaded( 'libzip' ) && isset( $zip_password ) && ! empty( $zip_password ) ) {
						$archive->set_zip_password( $file_name, $zip_password );
					}

					$archive->close();
				} else {
					return false;
				}

				return $file_url;
			}
		}
	}
}
